from typing import Optional

from dataclasses import field, dataclass
from ai_harness import harnessutils as aiutils

from ai_transformersx.model import *

log = aiutils.getLogger('task')


@dataclass
class ModelArguments:
    """
    Arguments pertaining to which model/config/tokenizer we are going to fine-tune from.
    """
    config_name: Optional[str] = field(
        default=None, metadata={"help": "Pretrained config name or path if not the same as model_name"}
    )

    model_base_dir: str = field(default="./models/pretrained",
                                metadata={"help": "the path base dir of models"})

    model_type: str = field(default="bert",
                            metadata={
                                "help": "the type of model: " + str(ModelType.names())})

    model_task_type: str = field(default="seq_cls",
                                 metadata={
                                     "help": "the task type of model: " + str()})

    model_mode: str = field(default="classification",
                            metadata={"help": "the model of model: " + str(ModelMode.names())})

    model_name: str = field(default="bert-base-chinese",
                            metadata={"help": "the name of model: " + str(ALL_TASK_MODEL_PATHS)})

    tokenizer_type: str = field(default="default",
                                metadata={"help": "the name of tokenizer: default,fast"})

    language: str = field(default="cn",
                          metadata={"help": "the language of model: cn, en"})

    freeze_parameter: str = field(default="bert", metadata={"help": "The parameter name for freeze"})

    num_labels: str = field(default=2,
                            metadata={"help": "the number of label"})

    def validate(self):
        if not self.model_base_dir:
            raise ValueError("model_base_dir can not be empty.")



from dataclasses import field, dataclass
from ai_harness import harnessutils as aiutils

log = aiutils.getLogger('task')


@dataclass
class DataArguments:
    """
    Arguments pertaining to what data we are going to input our model for training and eval.

    Using `HfArgumentParser` we can turn this class
    into argparse arguments to be able to specify them on
    the command line.
    """
    task_name: str = field(default="",
                           metadata={
                               "help": "the task name."}
                           )

    model_mode_for_data: str = field(default="classification",
                                     metadata={"help": "the model of model: classification or regression"})

    ##todo: refactor to use the max_position_embeddings of the config of model
    max_seq_length: int = field(
        default=512,
        metadata={
            "help": "The maximum total input sequence length after tokenization. Sequences longer "
                    "than this will be truncated, sequences shorter will be padded."
        },
    )
    predict: bool = field(
        default=False, metadata={"help": "Overwrite the cached training and evaluation sets"}
    )
    overwrite_cache: bool = field(
        default=False, metadata={"help": "Overwrite the cached training and evaluation sets"}
    )
    progress_bar: bool = field(default=True, metadata={"help": "Whether shows the progress_bar"})



from collections import OrderedDict

from .electra.modelingx_electra import ElectraForSequenceClassification
from .bert.models import Bert_Task_Models
from .albert.models import Albert_Task_Models
from .electra.models import Electra_Task_Models
from .roberta.models import Roberta_Task_Models
from .xlnet.models import XLNet_Task_Models
from .model_utils import ModelType, ModelTaskType, ModelMode

ALL_TASK_MODELS = OrderedDict([
    (ModelType.albert, Albert_Task_Models()),
    (ModelType.bert, Bert_Task_Models()),
    (ModelType.roberta, Roberta_Task_Models()),
    (ModelType.electra, Electra_Task_Models()),
    (ModelType.xlnet, XLNet_Task_Models())
])

ALL_TASK_MODEL_PATHS = [task_model.MODEL_PATHS for task_model in ALL_TASK_MODELS.values()]


def task_model(model_type, model_path, model_task_type, tokenizer_name='default', language='cn'):
    models = ALL_TASK_MODELS.get(model_type)
    if not models:
        raise ValueError("Cannot find models with model type {}".format(model_type))
    return models.task_model(model_path, model_task_type, tokenizer_name, language)



import os
import sys

from dataclasses import field, dataclass
from ai_harness import harnessutils as aiutils
from transformers import HfArgumentParser

from ai_transformersx.model.model_args import ModelArguments
from ai_transformersx.data.data_args import DataArguments
from ai_transformersx.train.training_args import TrainingArguments

log = aiutils.getLogger('task')


@dataclass
class TaskArguments:
    model_args: ModelArguments
    data_args: DataArguments
    training_args: TrainingArguments


def parse_args(*extensions) -> (TaskArguments,):
    parser = HfArgumentParser((ModelArguments, DataArguments, TrainingArguments) + extensions)

    if len(sys.argv) == 2 and sys.argv[1].endswith(".json"):
        # If we pass only one argument to the script and it's the path to a json file,
        # let's parse it to get our arguments.
        args = parser.parse_json_file(json_file=os.path.abspath(sys.argv[1]))
    else:
        args = parser.parse_args_into_dataclasses()

    if not args[2].output_dir:
        args[2].output_dir = 'models/finetuning'
    task_args = TaskArguments(args[0], args[1], args[2])
    return task_args, tuple(args[3:])
